<?php
/**
 * dtweb - Device Tree editor for UDOO boards
 * Copyright (C) 2015 Francesco Montefoschi <francesco.monte@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package dtweb
 * @author  Francesco Montefoschi
 * @license http://www.gnu.org/licenses/gpl-3.0.html  GNU GPL 3.0
 */

class Controller_Save extends Controller
{
    public function run()
    {
	$origDtsDir = realpath(__DIR__ . '/../../dts');
	$url = parse_url($_SERVER['HTTP_REFERER']);
	parse_str($url['query'], $params);
	$kernelDir = $params['kerneldir'];
	$board = $params['board'];

        switch ($board) {
            case 'Armadillo-640':
                $orig = "$origDtsDir/armadillo-640-expansion-interface.dtsi";
                $dtsi = "$kernelDir/arch/arm/boot/dts/armadillo-640-expansion-interface.dtsi";
                $target = "armadillo-640-at-dtweb.dtb";
                $cores = 1;
                $flags = "ARCH=arm CROSS_COMPILE=arm-linux-gnueabihf-";
                break;
            case 'Armadillo-610':
                $orig = "$origDtsDir/armadillo-610-expansion-interface.dtsi";
                $dtsi = "$kernelDir/arch/arm/boot/dts/armadillo-610-expansion-interface.dtsi";
                $target = "armadillo-610-at-dtweb.dtb";
                $cores = 1;
                $flags = "ARCH=arm CROSS_COMPILE=arm-linux-gnueabihf-";
                break;
            default:
                $this->json(array('success' => true, 'message' => "Invalid board model: " . $_POST['id']));
        }
        
        $this->build($orig, $dtsi, $target, $cores, $flags);
	}
    
    private function build($orig, $dtsi, $target, $cores, $flags)
    {
        $kernelDir = Service_KernelDetector::kernelFromURL();

        $configDir = $kernelDir;
        
        $dtreader = new Service_DeviceTreeReader();
        $dt = $dtreader->parse($orig);

        $conf = json_decode($_POST['conf'], true);
        $prop = json_decode($_POST['prop'], true);
        $snvspins = array();
        if (isset($_POST['snvspins'])) {
            $snvspins = json_decode($_POST['snvspins'], true);
        }
        $config = new Model_Configuration();
        foreach ($conf as $name => $pins) {
            $config->add($name, $pins);
        }

        $dteditor = new Service_DeviceTreeEditor($dt);
        $dteditor->setBoardType($_POST['id']);
        $dteditor->disableEverything();
        $dteditor->applyConfiguration($config, $prop, $snvspins);

        file_put_contents($dtsi, $dteditor->generate());

        /* remove dummy node */
        exec("bash -c 'sed -i \"/DUMMY_NODES_START/,/DUMMY_NODES_END/d\" $dtsi'");
        /* remove comment line */
        exec("bash -c 'sed -i \"/^\/\//d\" $dtsi'");
        /* combine multiple newlines */
        exec("bash -c 'sed -i \"/^$/{N; /^\\n$/D;}\" $dtsi'");
        /* remove simbol of dtweb */
        exec("bash -c 'sed -i \"s/{{.\+}}//g\" $dtsi'");
        /* remove discarded comment */
        exec("bash -c 'sed -i \"s/\/\/ $//g\" $dtsi'");
        /* remove trailing whitespace */
        exec("bash -c 'sed -i \"s/[ \\t]*$//g\" $dtsi'");

        chdir("$kernelDir/arch/arm/boot/dts");
        exec("bash -c 'rm -f $target'");
        
        chdir($kernelDir);
        exec("bash -c 'make $flags -j$cores $target 2>&1'", $output, $returnCode);
        
        if ($returnCode != 0) {
            $this->json(array(
                'success' => false,
                'message' => 'Cannot build DTB! Error log: <br><br><code>' . implode('<br>', $output) . '</code>'
            ));
        }
        
        file_put_contents("$configDir/.dtweb." . $_POST['id'] . ".conf", $_POST['conf']);
        file_put_contents("$configDir/.dtweb.prop." . $_POST['id'] . ".conf", $_POST['prop']);

        $this->json(array('success' => true));
    }
}
